/* $Id: dns.c,v 1.1 2000/04/07 14:28:30 bind Exp $ */

#include "sentinel.h"

char dnsname[255];

struct udp_header {
	u_int16_t uh_sport;		/* source port */
	u_int16_t uh_dport;		/* destination port */
	u_int16_t uh_ulen;		/* udp length */
	u_int16_t uh_sum;		/* udp checksum */
};

struct dns {
  unsigned short int id;
  unsigned char   rd:1;		/* recursion desired */
  unsigned char   tc:1;		/* truncated message */
  unsigned char   aa:1;		/* authoritive answer */
  unsigned char   opcode:1;	/* purpose of message */
  unsigned char   qr:1;		/* response flag */
  unsigned char   rcode:1;	/* response code */
  unsigned char   unused:1;	/* unused bits */
  unsigned char   pr:1;		/* primary server required (non standard) */
  unsigned char   ra:1;		/* recursion available */
  unsigned short int que_num;
  unsigned short int rep_num;
  unsigned short int num_rr;
  unsigned short int num_rrsup;
};

int processdns(char *);
void nameformat(char *, char *);
int dnsgen(char *);

int dnstest(void)
{
  pcap_t *socket;
  char *pkt;
  struct bpf_program prog;
  struct pcap_pkthdr pcap_h;

  unsigned int network, netmask;

  pcap_lookupnet(device, &network, &netmask, errbuf);

  socket = pcap_open_live(device, 128, 1, 1024, errbuf);
  if(socket == NULL) {
    perror("pcap_open_live");
    exit(1);
  }
 
  pcap_compile(socket, &prog, "udp dst port 53", 0, netmask);
  
  pcap_setfilter(socket, &prog);
  pkt = (u_char *)pcap_next(socket, &pcap_h);

  if(pkt != NULL)
  
    if(processdns(pkt) == 1) { 
      pcap_close(socket); 
      return 1; }
 
    pcap_close(socket);
    return 0;
  
}
int processdns(char *pkt)
{
  struct ip *ip;
  struct udp_header *udp;
  struct dns *dns;
  char *data, *host;
  char names[100];
  int count;

  ip = (struct ip *) (pkt + offset);
  udp = (struct udp_header *) (pkt + offset + IPHDRSIZE);
  dns = (struct dns *) (pkt + offset + IPHDRSIZE + UDPHDRSIZE);
  data = (char *) (pkt + offset + IPHDRSIZE + UDPHDRSIZE + DNSHDRSIZE);

  memset(names,'\0',sizeof(names));
   
  if(ip->ip_p == 17) {
    if(ntohs(udp->uh_dport) == 53) {
      host = inet_ntoa(ip->ip_src);
      if(strstr(host,dst) == NULL) return 0;
#ifdef DEBUG
      printf("\n\ndns id: %d\n", ntohs(dns->id));
      printf("response flag: %d\n", dns->qr);
      printf("response code: %d\n",dns->rcode);
      printf("opcode: %d\n", dns->opcode);
      printf("recursion: %d\n", dns->rd);
      printf("truncated message: %d\n", dns->tc);
      printf("authoritive answer: %d\n", dns->aa);
      printf("count: %d\n", htons(dns->que_num));
#endif

  if(!dns->que_num) return 0;

      count = (char)*data;

      while(count) { 
        (char *)data++;
        strncat(names, (char *)data, count);
        data += count;
        count = (char)*data;
        strncat(names,".",sizeof(names) - strlen(names));
      }

      names[strlen(names)-1] = '\0';

      if(strstr(names,dnsname) != NULL) { 
#ifdef DEBUG
      printf("\nResolve: %s Size = %d\n",names,strlen(names));
      printf("Match: %s Size = %d",dnsname, strlen(dnsname));
#endif
     return 1;
     }
    }
  }
  return 0;
}

int dnsgen(char *host)
{
  struct sockaddr_in test;
  char *ip;
  int A1, A2, A3, A4;

  if(!inet_aton(host,&test.sin_addr)) {
    printf("You must specify a valid ip for the fake host\n");
    exit(1);
  }

  ip = strtok(host,".");
  A1 = atoi(ip);
  ip = strtok(NULL,".");
  A2 = atoi(ip);
  ip = strtok(NULL,".");
  A3 = atoi(ip);
  ip = strtok(NULL,".");
  A4 = atoi(ip);

  sprintf(dnsname,"%d.%d.%d.%d.in-addr.arpa",A4,A3,A2,A1);
  return 0;
}

